`default_nettype none
`timescale 1ns/1ps
// }}}
module	sddma_s2mm_axi #(
		// {{{
		parameter	ADDRESS_WIDTH=32,
		parameter	BUS_WIDTH = 32,
		parameter [0:0]	OPT_LITTLE_ENDIAN = 1'b1,
		parameter	LGPIPE = 10,
		// Abbreviations
		localparam	DW = BUS_WIDTH,
		localparam	AW = ADDRESS_WIDTH
		// }}}
	) (
		// {{{
		input	wire	i_clk, i_reset,
		// Configuration
		// {{{
		input	wire			i_request,
		output	reg			o_busy, o_err,
		input	wire			i_inc,
		input	wire	[1:0]		i_size,
		// input wire	[LGLENGTH:0]	i_transferlen,
		input wire [ADDRESS_WIDTH-1:0]	i_addr,	// Byte address
		// }}}
		// Incoming Stream interface
		// {{{
		input	wire			S_VALID,
		output	wire			S_READY,
		input	wire	[DW-1:0]	S_DATA,
		// How many bytes are valid?
		input	wire [$clog2(DW/8):0]	S_BYTES,
		input	wire			S_LAST,

		output	wire			M_AXI_AWVALID,
		input	wire			M_AXI_AWREADY,
		output	wire	[AW-1:0]	M_AXI_AWADDR,
		//
		output	wire			M_AXI_WVALID,
		input	wire			M_AXI_WREADY,
		output	wire	[DW-1:0]	M_AXI_WDATA,
		output	wire	[DW/8-1:0]	M_AXI_WSTRB,
		output	wire			M_AXI_WLAST,
        //
        input	wire			M_AXI_BVALID,
		output	wire			M_AXI_BREADY,
		input	wire	[1:0]		M_AXI_BRESP
		// }}}
		// }}}
	);

    // 1. When we receive a request, we need to load the requested address and then
    // raise S_READY
    // 2. when S_READY and S_VALID are both high, we assert both AWVALID, WVALID. and lower S_READY
    // 3. When the appropriate READY is seen, we lower both, increase addr and raise S_READY again
    // repeat 2-3 repeatedly until we see last

	reg [31:0] r_addr;
	reg r_ready;
	reg r_wvalid;
	reg r_awvalid;
	reg [31:0] r_wdata;

	assign S_READY = r_ready;
	assign M_AXI_AWADDR = r_addr;
	assign M_AXI_WVALID = r_wvalid;
	assign M_AXI_WSTRB = 4'b1111;
	assign M_AXI_AWVALID = r_awvalid;
	assign M_AXI_WDATA = r_wdata;
	assign M_AXI_BREADY = 1'b1;

	always @(posedge i_clk) begin
		if (i_reset) begin
			r_wvalid <= 1'b0;
			r_awvalid <= 1'b0;
		end

		if (i_request) begin
			r_addr <= i_addr;
			r_ready <= 1'b1;
		end else begin
			if (r_ready && S_VALID) begin
				r_wvalid <= 1'b1;
				r_awvalid <= 1'b1;
				r_wdata <= S_DATA;

				r_ready <= 1'b0;
			end

			if (~r_ready) begin
				if (M_AXI_AWREADY) begin
					r_awvalid <= 1'b0;
				end

				if (M_AXI_WREADY) begin
					r_wvalid <= 1'b0;
				end

				if (~r_awvalid && ~r_wvalid) begin
					r_addr <= r_addr + 4;
					r_ready <= 1'b1;
				end
			end
		end
	end

endmodule